(
	/*
	NAME: makeTX batcher script
	GOALS: 
	*) collect textures from the user's selected folder and subfolders
	*) Build a batch of the commands
	*) Execute the batch and convert the input textures to .TX
	*/
	
	-- Globals
	global txConverterRoll
	try (destroyDialog txConverterRoll) catch()
	
	-- Locals
	mapFilePaths=#()	--this array will contain the textures
	mapExtensions=#("*.bmp","*.cin","*.dds","*.dpx","*.exr","*.hdr","*.jpg","*.jpeg","*.png","*.ptx","*.rla","*.sgi","*.pic","*.tga","*.tif","*.tiff")	--this array contains the string patterns for the texture extensions. More can be added, or some removed, compatibly with makeTX.exe. Check "https://github.com/OpenImageIO/oiio/blob/release/src/doc/openimageio.pdf" for more info.
	rootFolder=undefined	--the root folder to search textures into. Initialises as "undefined", and cancelling out of the folder picking will set it again to "undefined".
	makeTXPath=undefined	--the makeTX.exe file path. Will be filled later if the file is found.
	optionsSet=#(" --hicomp"," --tile 64 64")		--array containing options, WITH leading space. f.e. #("--hicomp" ). Off currently.
	totCounter = 0 		--total file counter
	
	fn findVRayBinFolder =		--quick and safe method to find out which V-Ray version is being currently used, and proceed to fidn the bin folder
	(
		Val1=undefined
		maxExtendedVersionNumber="20" + ((maxVersion())[4]-2) as string 
		try
		(
			registry.openKey HKEY_LOCAL_MACHINE "SYSTEM" accessRights:#readOnly key:&key1
			registry.openKey key1 "CurrentControlSet" accessRights:#readOnly key:&key2
			registry.openKey key2 "Control" accessRights:#readOnly key:&key3
			registry.openKey key3 "Session Manager" accessRights:#readOnly key:&key4
			registry.openKey key4 "Environment" accessRights:#readOnly key:&key5
			registry.queryValue key5 ("VRAY4_FOR_3DSMAX" + maxExtendedVersionNumber+"_MAIN") type:#REG_EXPAND_SZ value:&Val1 expand:true 
		)
		catch
		(
			undefined
		)
		Val1
	)
	
	fn getmakeTX =	--function to find the path to the makeTX.exe executable file
	(
		makeTXPath=undefined
		binPath=findVRayBinFolder()		--first it finds the v-ray bin folder
		if binPath != undefined then 	--if it made sense of the current install
		(
			if doesFileExist (binPath+@"\makeTX.exe") then	--if the file actually exists
			(
				makeTXPath = binPath+@"\makeTX.exe" 		--it will build and store the right path
			)
			else
			(
				print "couldn't find makeTX.exe in the bin path of the V-Ray install!"	--else it'll warn the user and leave the path undefined
			)
		)
		else
		(
			print "Couldn't find the bin folder for the current V-Ray install!"	--else it will warn the user the path to the bin folder couldn't be constructed
		)
		makeTXPath
		
	)
	
	fn getFilesRecursive root pattern =			--function to recursively parse a folder and subfolders to look for specific file types
	(
		root+="/"
		dir_array = #(root)
		join dir_array (GetDirectories (root+"/*") recurse:true)
		my_files = #()
		for f in dir_array do
		  join my_files (getFiles (f + pattern))
		my_files
	)	
	
	fn collectBitmapTextures rootFolder extensions =
	(
		mapFilePaths=#()
		for ex in extensions do
		(
			join mapFilePaths (getFilesRecursive rootFolder ex)
		)
		mapFilePaths
	)
	
	fn setRootFolder =
	(
		rootFolder = getSavePath caption:"Select a root folder to parse for textures" initialDir:"$maps"
-- 		rootFolder = getSavePath caption:"Select a root folder to parse for textures" initialDir:@"D:\_Maps\environments\all_probes"	
	)
	
	fn mainFunction =
	(

-- 		progressStart "Converting files to tiled .TX"
		for f=1 to totCounter do 
		(
			windows.processPostedMessages()
			baseCmd = (makeTXPath + " " + mapFilePaths[f]) 
-- 			for o in optionsSet do baseCmd += o
			percent = ((f as float / totCounter)*100.)
			print ("Converting " + mapFilePaths[f] + "; number "+ f as string + " of " + totCounter as string + "; " + percent as string + "% done.")
			txConverterRoll.pb1.value = percent
			txConverterRoll.lbl2.text = (percent as string + "% done")
			vrayshellLaunch baseCmd waitExec:true
		)
-- 		progressEnd()
	)
	
	
	rollout txConverterRoll ".TX converter" width:368 height:280
	(
		button 'btn1' "Pick Root to scan for textures" pos:[8,8] width:352 height:40 align:#left
		label 'lbl1' "Info" pos:[8,56] width:352 height:96 align:#left
		
		button 'btn2' "Go" pos:[8,160] width:344 height:64 enabled:false align:#left

		progressBar 'pb1' "ProgressBar" pos:[8,256] width:344 height:16 color:(color 0 255 0) align:#left
		label 'lbl2' "" pos:[8,232] width:344 height:16 align:#left
		
		on txConverterRoll open do
		(
			makeTXPath = getmakeTX()
			if makeTXPath == undefined do 
			(
				messageBox "Couldn't get to MakeTX.exe, aborting."
				destroydialog txConverterRoll
			)
		)
		on btn1 pressed do
		(
			setRootFolder()
			if rootFolder!=undefined then
			(
				btn1.text = rootFolder
				lbl1.text="Scanning the root and subfolders for texture files.\nPlease be patient."
				collectBitmapTextures rootFolder mapExtensions
				totCounter = mapFilePaths.count
				if totCounter != 0 then
				(
					lbl1.text = ("Found " + totCounter as string + " texture files.")
					btn2.enabled=true
				)
				else
				(
					lbl1.text = ("Found no texture files. Please choose a different folder.")
					btn2.enabled=false
				)
			)
			else
			(
				btn1.text="Pick Root to scan for textures"
				lbl1.text=""
				btn2.enabled=false
			)
		)
		on btn2 pressed do
		(
			mainFunction()
			lbl1.text="Conversion attempt completed.\nThe MaxScript Listener will have some more information.\nPlease verify the results!"
			lbl2.text=""
			btn2.enabled=false
		)
	)
	
	createdialog txConverterRoll
)